/* This file implements a classic producer consumer program using
   pthreads */
#include <stdlib.h>
#include <pthread.h>
#include <sched.h>


#include "hcu_headers.h"



#define DEBUG 0
#if (DEBUG == 1)
#define dprintf(format, args...) printf(format, ##args)
#else
#define dprintf(format, args...) do {} while (0)
#endif

void *producer_thread_entry(void *arg);
void *producer_thread(void *arg);
void *consumer_thread_entry(void *arg);
void *consumer_thread(void *arg);

pthread_mutex_t lock;

int main()
{
  pthread_t producer_thread_id;
  pthread_t consumer_thread_id;

  pthread_mutex_init(&lock, NULL);

  pthread_create(&producer_thread_id, NULL, producer_thread_entry, NULL);
  pthread_create(&consumer_thread_id, NULL, consumer_thread_entry, NULL);

  while (1) {
    sleep(1);
  }

  return 0;
}



void exit_on_iterations(long iteration_counter, long max)
{
  if (iteration_counter >= max) {
    printf("iteration counter = %ld\n", iteration_counter);
    exit(0);
  }
}


void *producer_thread_entry(void *arg)
{
  producer_thread(arg);

  return NULL;
}


void *producer_thread(void *arg)
{
  long producer_local_variable = 55555;
  long producer_counter = 0;
  pthread_t thread_id;

  thread_id = pthread_self();
  while (1) {
    pthread_mutex_lock(&lock);
    producer_counter++;
    dprintf("HCU Thread %d producing(%ld,%ld)...\n", thread_id, producer_counter, producer_local_variable);
    pthread_mutex_unlock(&lock);
    /* Sleep 0.1 seconds */
    usleep(100000);
    sched_yield();
  }
}



void *consumer_thread_entry(void *arg)
{
  consumer_thread(arg);

  return NULL;
}



void *consumer_thread(void *arg)
{
  long consumer_local_variable = 66666;
  long consumer_counter = 0;
  pthread_t thread_id;

  thread_id = pthread_self();
  while (1) {
    pthread_mutex_lock(&lock);
    consumer_counter++;
    exit_on_iterations(consumer_counter, 3);
    dprintf("HCU Thread_v2 %d consuming(%ld,%ld)...\n", thread_id, consumer_counter, consumer_local_variable);
    pthread_mutex_unlock(&lock);
    /* Sleep 0.1 seconds */
    usleep(100000);
    sched_yield();
  }
}
